@@ -1,18 +1,8 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2018-2021 Rumma & Ko Ltd
 # License: GNU Affero General Public License v3 (see file COPYING for details)
+# Source documentation see https://react.lino-framework.org/dev
 
-"""
-A user interface for Lino applications that uses FaceBooks React JS framework.
-
-
-.. autosummary::
-   :toctree:
-
-    views
-    renderer
-    models
-"""
 
 from django.utils import translation
 from lino.core import constants
@@ -24,7 +14,8 @@ class Plugin(Plugin):
     ui_handle_attr_name = 'react_handle'
 
     # needs_plugins = ['lino.modlib.jinja', 'lino.modlib.memo']
-    needs_plugins = ['lino.modlib.system', 'lino.modlib.jinja']
+    # needs_plugins = ['lino.modlib.system', 'lino.modlib.jinja']
+    needs_plugins = ['lino.modlib.jinja']
     # disables_plugins = ['tinymce', 'extensible']
 
     url_prefix = 'react'
@@ -33,6 +24,9 @@ class Plugin(Plugin):
     support_async = True
     top_paginator = True
 
+    # resizable_panel = False
+    resizable_panel = True
+
     # media_base_url = "http://maxcdn.bootstrapcdn.com/bootstrap/3.2.0/"
 
     def on_ui_init(self, kernel):
@@ -86,6 +80,11 @@ class Plugin(Plugin):
 
             url(rx + r'api/main_html$', views.MainHtml.as_view()),
 
+            url(rx + r'auth/smart_id/callback$', views.SmartId.as_view()),
+            url(rx + r'auth/smart_id$', views.SmartIdEntry.as_view()),
+
+            url(rx + r'api/top_links$', views.TopLinks.as_view()),
+
             path('dashboard/<int:index>', views.DashboardItem.as_view()),
 
             # To be fased out
@@ -98,24 +97,27 @@ class Plugin(Plugin):
                 views.ApiList.as_view()),
             url(rx + r'api/(?P<app_label>\w+)/(?P<actor>\w+)/(?P<pk>[^/]+)$',
                 views.ApiElement.as_view()),
+            url(rx + r'api/(?P<app_label>\w+)/(?P<actor>\w+)/(?P<pk>[^/]+)/(?P<field>\w+)$',
+                views.TextField.as_view()),
             # url(rx + r'api/(?P<app_label>\w+)/(?P<actor>\w+)/(?P<pk>[^/]+)/(?P<field>\w+)/suggestions$',
             #     views.Suggestions.as_view()),
             url(rx + r'values/(?P<app_label>\w+)/(?P<actor>\w+)/(?P<pk>.+)/(?P<field>.+)$',
                 views.DelayedValue.as_view()),
             url(rx + r'choices/(?P<app_label>\w+)/(?P<actor>\w+)$',
                 views.Choices.as_view()),
-            url(rx + r'choices/(?P<app_label>\w+)/(?P<actor>\w+)/'
-                     '(?P<field>\w+)$',
+            url(rx + r'choices/(?P<app_label>\w+)/(?P<actor>\w+)/(?P<field>\w+)$',
                 views.Choices.as_view()),
-            url(rx + r'apchoices/(?P<app_label>\w+)/(?P<actor>\w+)/'
-                     '(?P<an>\w+)/(?P<field>\w+)$',
-                views.ActionParamChoices.as_view()),
+            url(rx + r'choices/(?P<app_label>\w+)/(?P<actor>\w+)/(?P<field>\w+)/(?P<an>\w+)$',
+                views.Choices.as_view()),
+            # url(rx + r'apchoices/(?P<app_label>\w+)/(?P<actor>\w+)/'
+            #          r'(?P<an>\w+)/(?P<field>\w+)$',
+            #     views.ActionParamChoices.as_view()),
             # For generating views
             # url(rx + r'callbacks/(?P<thread_id>[\-0-9a-zA-Z]+)/'
             #          '(?P<button_id>\w+)$',
             #     views.Callbacks.as_view()),
             #
-            url(rx+ r'choicelists/',
+            url(rx + r'choicelists/',
                 views.ChoiceListModel.as_view()),
 
         ]
@@ -130,7 +132,7 @@ class Plugin(Plugin):
 
     def get_used_libs(self, html=False):
         if html is not None:
-            yield ("React", '16.6', "https://reactjs.org/")
+            yield ("React", '18.3.1', "https://reactjs.org/")
 
-    # def get_requirements(self, site):
-    #     yield 'lino_react'
+    def get_requirements(self, site):
+        yield 'lino_react'
